.nolist
    include     masm32rt.inc    ; defaults to .386
    .586
    include     timers.asm      ; requires a pentium
.listall

    LoopCount equ 100000;0

    Procs TEXTEQU <szCatStr,hutchszCat2,kuntz2>
    MaxStr equ 2000     ; largest string to be used in tests
    Tests TEXTEQU <0/0, 1/0, 0/1, 1/1, 5/5, 13/17, 32/16, 223/271, 251/1031, 1031/251>
    
    %FOR proc,<Procs>
        proc proto :dword,:dword
    ENDM

    MAXWIDTH  equ 20
    HDRWIDTH  equ 12
    COLWIDTH  equ 6

.data

align 16
String1 db MaxStr*2+64 dup (0)      ; what will be input string1 to routines.  The extra 64 bytes are to represent
align 16                            ;     a possible next string in the real world that we would not want wiped out.
String2 db MaxStr+32   dup (0)      ; what will be input string2 to routines    

align 16
Str1Offset      dd  0   ; offset to starting byte within String1 based upon number of extra bytes being tested
Str2Offset      dd  0   ; offset to starting byte within String2  etc.
Str1ZeroOffset  dd  0   ; computed offset to zero byte terminator of first string for each test
Str2ZeroOffset  dd  0   ; computed offset to zero byte terminator of second string
ErrorFound      dd  0   ; flag that an error was found processing a proc
.code
align 16

;Printing routines from Phil
sxFld proc DDpointer:DWORD, rvstring:DWORD, DDwidth:DWORD, DDalign:DWORD    
    .if DDalign == 0                    ; align left?
        invoke szLeft,DDpointer,rvstring,DDwidth
        invoke szLen,rvstring
        mov edx,rvstring
    @@: cmp eax,DDwidth
        jae @F
        mov byte ptr [edx+eax],32
        inc eax
        jmp @B
    @@: mov byte ptr [edx+eax],0
    .elseif DDalign == 1                ; align right?
        invoke szLen,DDpointer
        mov edx,rvstring
        xor ecx,ecx
    @@: cmp eax,DDwidth
        jae @F
        mov byte ptr [edx+ecx],32
        inc eax
        inc ecx
        jmp @B
    @@: add edx,ecx
        invoke szLeft,DDpointer,edx,DDwidth
    .endif
    ret 4*4
sxFld endp

fld$ MACRO DDpointer,DDwidth,DDalign:=<0>       ; Default is align left
    LOCAL rvstring
    .data
        rvstring db MAXWIDTH+4 dup (0)
        align 16
    .code
    invoke sxFld,reparg(DDpointer),ADDR rvstring,DDwidth,DDalign
    EXITM <ADDR rvstring>
ENDM

CheckForError proc T1:dword,T2:dword    ; check result string for correctness.  return 0=ok, 1=error
    mov esi,Str1Offset

    mov ecx,T1              ; there should first be T1 bytes of 255
    cmp ecx,0
    je Skip1                ; zero length string, skip this section
@@: lodsb
    cmp al,255
    jne Error1Found
    loop @b

Skip1:
    mov ecx,T2              ; there should now be T2 bytes of 254
    cmp ecx,0
    je Skip2                ; zero length string, skip this section
@@: lodsb
    cmp al,254
    jne Error2Found
    loop @b

Skip2:  
    lodsb                   ; there should now be the zero byte terminator
    cmp al,0
    jne Error3Found

    mov ecx,32              ; there should be at least 32 non-zero bytes following which could represent 
@@: lodsb                   ;   a string following our output string in the real world
    and al,254              ; accept either 254 or 255
    cmp al,254
    jne Error4Found
    loop @b

    clc ; all ok
    jmp CheckForErrorExit

Error1Found:
    print chr$(13,10,"Error 1 in routine")
    jmp BadError

Error2Found:
    print chr$(13,10,"Error 2 in routine")
    jmp BadError

Error3Found:
    print chr$(13,10,"terminator missing in routine")
    jmp BadError

Error4Found:
    print chr$(13,10,"Error 4 in routine")

BadError:
    stc ; skip testing for rest of this proc

CheckForErrorExit:
    ret
CheckForError EndP

align 16
start:
    
    print chr$(13,10)
    print fld$(chr$("String1 Size"),HDRWIDTH)

    %FOR T1T2,<Tests>
        sep instr 1,<T1T2>,</>
        T1 substr <T1T2>,1,sep-1    ; len of string 1
        T2 substr <T1T2>,sep+1      ; len of string 2
        print fld$(left$(chr$("&T1T2&"),sep-1),COLWIDTH,1)
    ENDM
    print chr$(13,10)
    print fld$(chr$("String2 Size"),HDRWIDTH)
    %FOR T1T2,<Tests>
        sep instr 1,<T1T2>,</>
        T1 substr <T1T2>,1,sep-1    ; len of string 1
        T2 substr <T1T2>,sep+1      ; len of string 2
        print fld$( remove$( chr$("&T1T2&"),left$(chr$("&T1T2&"),sep)),COLWIDTH,1)
    ENDM
    print chr$(13,10)

    print fld$(chr$("====================="),HDRWIDTH)
    %FOR T1T2,<Tests>
        print fld$(" =====================",COLWIDTH)
    ENDM
    print chr$(13,10)

    %FOR extra,<0,1,2,3>
        print chr$(13,10)
        print chr$("&extra& byte misalignment",13,10)
       %FOR proc,<Procs>
            print fld$(chr$("&proc&"),HDRWIDTH)
            %FOR T1T2,<Tests>
                sep instr 1,<T1T2>,</>
                T1 substr <T1T2>,1,sep-1    ; len of string 1
                T2 substr <T1T2>,sep+1      ; len of string 2

                ; create fresh test strings for each routine to use 
                mov eax,offset String1
                add eax,extra               ; start of string1 for testing
                mov Str1Offset,eax
                add eax,T1                  ; where zero byte terminator will be
                mov Str1ZeroOffset,eax
                
                mov eax,offset String2
                add eax,extra               ; start of string2 for testing
                mov Str2Offset,eax
                add eax,T2                  ; where zero byte terminator will be
                mov Str2ZeroOffset,eax
                
                mov edi,offset String1      ; create test string values
                mov ecx,MaxStr*2+48         ; leave 16 bytes at end for accidents
                mov al,255
                rep stosb
                mov eax,Str1ZeroOffset
                mov [eax],byte ptr 0        ; set length of string
                
                mov edi,offset String2      ; create test string values
                mov ecx,MaxStr+16
                mov al,254
                rep stosb
                mov eax,Str2ZeroOffset
                mov [eax],byte ptr 0        ; set length of string
                
                ; run twice to verify operation before running a million times
                
                mov eax,Str1ZeroOffset
                mov [eax],byte ptr 0        ; reset length of string
                invoke proc,Str1Offset,Str2Offset
                invoke CheckForError,T1,T2
                jnc @f  ; ok
                print chr$("&proc&",13,10)
                jmp QuitTests
@@:
                mov eax,Str1ZeroOffset
                mov [eax],byte ptr 0        ; reset length of string
                invoke proc,Str1Offset,Str2Offset
                invoke CheckForError,T1,T2
                jnc @f  ; ok
                print chr$("&proc&",13,10)
                jmp QuitTests
@@:
                ; now do actual time test
                counter_begin LoopCount, HIGH_PRIORITY_CLASS
                    mov eax,Str1ZeroOffset
                    mov [eax],byte ptr 0        ; reset length of string
                    invoke proc,Str1Offset,Str2Offset
                counter_end
                mov ebx,eax
                print fld$(sstr$(ebx),COLWIDTH,1)
                ; and one more check to be sure everything is still ok
                mov eax,Str1ZeroOffset
                mov [eax],byte ptr 0        ; reset length of string
                invoke proc,Str1Offset,Str2Offset
                invoke CheckForError,T1,T2
                jnc @f  ; ok
                print chr$("&proc&",13,10)
                jmp QuitTests
@@:
            ENDM
            print chr$(13,10)
        ENDM        
    ENDM
QuitTests:
    mov   eax,input(13,10,"Press enter to exit...")
    exit

; --------------------- Routines under test ---------------------------------------------
OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE
align 16
hutchszCat2 proc src:DWORD,szadd:DWORD

    push edi

    xor eax, eax
    mov edx, [esp+8]    ; src
    mov edi, [esp+12]   ; szadd
    sub edx, 1
    xor ecx, ecx

  align 4
  @@:
    add edx, 1
    cmp BYTE PTR [edx], 0
    jne @B

  align 4
  @@:
    mov al, [edi+ecx]
    mov [edx+ecx], al
    add ecx, 1
    test al, al
    jnz @B

    mov eax, [esp+8]    ; src   return start address of source

    pop edi

    ret 8
hutchszCat2 endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef
;-------------------------------------------------------------------------------
OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE
align 16
kuntz2 proc xstr1:dword, xstr2:dword
    mov dword ptr [esp-4], ebx      ; push ebx
    mov dword ptr [esp-8], edi      ; push edi
    mov edi, dword ptr [esp+4]      ; xstr1
    align 4
d1: mov edx, dword ptr [edi]
    mov ecx, edx
    add edi, 4
    sub edx, 1010101h   ;16843009
    and edx, 80808080h  ;2155905152
    jz d1
    not ecx
    and edx, ecx
    jz d1
    test dl, dl
    jnz d2
    test dh, dh
    jnz d3
    test edx, 0FF0000h  ; 16711680
    jnz d4
    jmp d5
d2: dec edi
d3: dec edi
d4: dec edi
d5: dec edi
    mov eax, dword ptr [esp+8]      ; xstr2
    align 4
d6: mov edx, dword ptr [eax]
    mov ecx, edx
    add eax, 4
    sub edx, 1010101h   ;16843009
    and edx, 80808080h  ;2155905152
    jnz d7
    mov dword ptr [edi], ecx
    add edi, 4
    jmp d6
d7: mov ebx, ecx
    not ecx
    and edx, ecx
    jnz d8
    mov dword ptr [edi], ebx
    add edi, 4
    jmp d6
d8: mov dword ptr [edi], ebx
    mov edi, dword ptr [esp-8]      ; pop edi
    mov ebx, dword ptr [esp-4]      ; pop ebx
    ret 8

kuntz2 endp

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef

;-------------------------------------------------------------------------------
end start
